@@ -17,9 +17,8 @@ from __future__ import annotations
 from collections.abc import Iterable, Sequence
 import inspect
 import operator
-from functools import partial
+from functools import partial, lru_cache
 from typing import Any, Callable, Type
-import warnings
 
 import numpy as np
 
@@ -28,14 +27,14 @@ from jax._src import dtypes
 from jax._src.abstract_arrays import numpy_scalar_types
 from jax._src.core import ShapedArray
 from jax._src.tree_util import (
-    PyTreeDef, tree_flatten, tree_unflatten, tree_map, tree_structure,
+    PyTreeDef, tree_flatten, tree_unflatten, tree_map,
     treedef_children, generate_key_paths, keystr, broadcast_prefix,
     prefix_errors)
 from jax._src.tree_util import _replace_nones
 from jax._src import linear_util as lu
 from jax._src.linear_util import TracingDebugInfo
 from jax._src.util import (safe_map, WrapKwArgs, Hashable, HashableFunction,
-                           Unhashable)
+                           Unhashable, safe_zip)
 from jax._src import traceback_util
 traceback_util.register_exclusion(__file__)
 
@@ -168,13 +167,8 @@ def _validate_argnums(sig: inspect.Signature, argnums: tuple[int, ...], argnums_
       return
 
   if argnums and (-min(argnums) > n_pos_args or max(argnums) >= n_pos_args):
-    # raise ValueError(f"Jitted function has {argnums_name}={argnums}, "
-    #                  f"but only accepts {n_pos_args} positional arguments.")
-    # TODO: 2022-08-20 or later: replace with error
-    warnings.warn(f"Jitted function has {argnums_name}={argnums}, "
-                  f"but only accepts {n_pos_args} positional arguments. "
-                  "This warning will be replaced by an error after 2022-08-20 "
-                  "at the earliest.", SyntaxWarning)
+    raise ValueError(f"Jitted function has {argnums_name}={argnums}, "
+                     f"but only accepts {n_pos_args} positional arguments.")
 
 _INVALID_KEYWORD_ARGUMENTS = (
   inspect.Parameter.POSITIONAL_ONLY,
@@ -210,30 +204,18 @@ def _validate_argnames(
       invalid_kwargs.add(param_name)
 
   # Check whether any kwargs are invalid due to position only
-  invalid_argnames = invalid_kwargs & set(argnames)
-  if invalid_argnames:
-    # raise ValueError(f"Jitted function has invalid argnames {invalid_argnames} "
-    #                  f"in {argnames_name}. These are positional-only")
-    # TODO: 2022-08-20 or later: replace with error
-    warnings.warn(f"Jitted function has invalid argnames {invalid_argnames} "
-                  f"in {argnames_name}. These are positional-only. "
-                  "This warning will be replaced by an error after 2022-08-20 "
-                  "at the earliest.", SyntaxWarning)
+  if invalid_argnames := (invalid_kwargs & set(argnames)):
+    raise ValueError(f"Jitted function has invalid argnames {invalid_argnames} "
+                     f"in {argnames_name}. These are positional-only")
 
   # Takes any kwargs
   if var_kwargs:
     return
 
   # Check that all argnames exist on function
-  invalid_argnames = set(argnames) - valid_kwargs
-  if invalid_argnames:
-    # TODO: 2022-08-20 or later: replace with error
-    # raise ValueError(f"Jitted function has invalid argnames {invalid_argnames} "
-    #                  f"in {argnames_name}. Function does not take these args.")
-    warnings.warn(f"Jitted function has invalid argnames {invalid_argnames} "
-                  f"in {argnames_name}. Function does not take these args."
-                  "This warning will be replaced by an error after 2022-08-20 "
-                  "at the earliest.", SyntaxWarning)
+  if invalid_argnames := (set(argnames) - valid_kwargs):
+    raise ValueError(f"Jitted function has invalid argnames {invalid_argnames} "
+                     f"in {argnames_name}. Function does not take these args.")
 
 
 def argnums_partial(f, dyn_argnums, args, require_static_args_hashable=True):
@@ -336,7 +318,9 @@ def _argnames_partial(fixed_kwargs: WrapKwArgs, *args, **dyn_kwargs):
   yield ans
 
 
-def donation_vector(donate_argnums, donate_argnames, args, kwargs) -> tuple[bool, ...]:
+@lru_cache(maxsize=4096)
+def donation_vector(donate_argnums, donate_argnames, in_tree,
+                    kws: bool = True) -> tuple[bool, ...]:
   """Returns a tuple with a boolean value for each leaf in args and kwargs.
 
   What if a user specifies donate_argnums but calls the function with kwargs
@@ -350,12 +334,17 @@ def donation_vector(donate_argnums, donate_argnames, args, kwargs) -> tuple[bool
   kwargs specified are donated.
   """
   res: list[bool] = []
-  for i, arg in enumerate(args):
+  if kws:
+    args_tree, kwargs_tree = treedef_children(in_tree)
+  else:
+    args_tree, kwargs_tree = in_tree, None
+  for i, arg in enumerate(args_tree.children()):
     donate = bool(i in donate_argnums)
-    res.extend((donate,) * tree_structure(arg).num_leaves)
-  for key, val in kwargs.items():
-    donate = key in donate_argnames
-    res.extend((donate,) * tree_structure(val).num_leaves)
+    res.extend((donate,) * arg.num_leaves)
+  if kwargs_tree is not None:
+    for key, val in safe_zip(kwargs_tree.node_data()[1], kwargs_tree.children()):  # type: ignore
+      donate = key in donate_argnames
+      res.extend((donate,) * val.num_leaves)
   return tuple(res)
 
 def rebase_donate_argnums(donate_argnums, static_argnums) -> tuple[int, ...]:
