@@ -14,14 +14,12 @@
 """Module for JAX callbacks."""
 from __future__ import annotations
 
-import dataclasses
 from collections.abc import Sequence
-import logging
+import dataclasses
 import functools
+import logging
 from typing import Any, Callable
 
-import numpy as np
-
 import jax
 from jax._src import core
 from jax._src import dispatch
@@ -33,9 +31,10 @@ from jax._src import util
 from jax._src.interpreters import ad
 from jax._src.interpreters import batching
 from jax._src.interpreters import mlir
-from jax._src.lib import xla_client as xc
 from jax._src.lax.control_flow.loops import map as lax_map
+from jax._src.lib import xla_client as xc
 from jax._src.sharding_impls import SingleDeviceSharding
+import numpy as np
 
 logger = logging.getLogger(__name__)
 
@@ -73,11 +72,14 @@ def pure_callback_impl(
     vectorized: bool,
 ):
   del sharding, vectorized, result_avals
-  try:
-    return callback(*args)
-  except BaseException:
-    logger.exception("jax.pure_callback failed")
-    raise
+  cpu_device, *_ = jax.local_devices(backend="cpu")
+  args = jax.device_put(args, cpu_device)
+  with jax.default_device(cpu_device):
+    try:
+      return tree_util.tree_map(np.asarray, callback(*args))
+    except BaseException:
+      logger.exception("jax.pure_callback failed")
+      raise
 
 
 pure_callback_p.def_impl(functools.partial(dispatch.apply_primitive,
@@ -232,7 +234,7 @@ def pure_callback_lowering(
       list(args),
       ctx.avals_in,
       ctx.avals_out,
-      False,
+      has_side_effect=False,
       sharding=op_sharding,
   )
   return result
@@ -255,14 +257,40 @@ def pure_callback(
     vectorized: bool = False,
     **kwargs: Any,
 ):
-  """Calls a pure Python callback.
+  """Calls a pure Python callback. Works under :func:`jit`/:func:`~vmap`/etc.
 
   For more explanation, see `External Callbacks`_.
 
+  ``pure_callback`` enables calling a Python function in JIT-ed JAX functions.
+  The input ``callback`` will be passed NumPy arrays in place of JAX arrays and
+  should also return NumPy arrays. Execution takes place on CPU, like any
+  Python+NumPy function.
+
+  The callback is treated as functionally pure, meaning it has no side-effects
+  and its output value depends only on its argument values. As a consequence, it
+  is safe to be called multiple times (e.g. when transformed by :func:`~vmap` or
+  :func:`~pmap`), or not to be called at all when e.g. the output of a
+  `jit`-decorated function has no data dependence on its value. Pure callbacks
+  may also be reordered if data-dependence allows.
+
+  When `vmap`-ed the behavior will depend on the value of the
+  ``vectorized`` keyword argument. When ``vectorized`` is ``True``, the callback
+  is assumed to obey
+  ``jax.vmap(callback)(xs) == callback(xs) == jnp.stack([callback(x) for x in xs])``.
+  Therefore, the callback will be called directly on batched inputs (where the
+  batch axes are the leading dimensions). Additionally, the callbacks should
+  return outputs that have corresponding leading batch axes. If not vectorized
+  ``callback`` will be mapped sequentially across the batched axis.
+  For example, if ``callback = lambda x, y: np.matmul(x, y)``, then we are free
+  to set ``vectorized=True`` because the ``np.matmul`` function handles
+  arbitrary leading batch dimensions.
+
   Args:
     callback: function to execute on the host. The callback is assumed to be a pure
       function (i.e. one without side-effects): if an impure function is passed, it
-      may behave in unexpected ways, particularly under transformation.
+      may behave in unexpected ways, particularly under transformation. The callable
+      will be passed PyTrees of arrays as arguments, and should return a PyTree of
+      arrays that matches ``result_shape_dtypes``.
     result_shape_dtypes: pytree whose leaves have ``shape`` and ``dtype`` attributes,
       whose structure matches the expected output of the callback function at runtime.
       :class:`jax.ShapeDtypeStruct` is often used to define leaf values.
@@ -299,75 +327,6 @@ def pure_callback(
   return tree_util.tree_unflatten(out_tree, out_flat)
 
 
-def pure_callback_api(
-    callback: Callable[..., Any],
-    result_shape_dtypes: Any,
-    *args: Any,
-    sharding: SingleDeviceSharding | None = None,
-    vectorized: bool = False,
-    **kwargs: Any,
-):
-  """Applies a functionally pure Python callable. Works under :func:`jit`/:func:`~pmap`/etc.
-
-  ``pure_callback`` enables calling a Python function in JIT-ed JAX functions.
-  The input ``callback`` will be passed NumPy arrays in place of JAX arrays and
-  should also return NumPy arrays. Execution takes place on CPU, like any
-  Python+NumPy function.
-
-  The callback is treated as functionally pure, meaning it has no side-effects
-  and its output value depends only on its argument values. As a consequence, it
-  is safe to be called multiple times (e.g. when transformed by :func:`~vmap` or
-  :func:`~pmap`), or not to be called at all when e.g. the output of a
-  `jit`-decorated function has no data dependence on its value. Pure callbacks
-  may also be reordered if data-dependence allows.
-
-  When :func:`~pmap`-ed, the pure callback will be called several times (one on each
-  axis of the map). When `vmap`-ed the behavior will depend on the value of the
-  ``vectorized`` keyword argument. When ``vectorized`` is ``True``, the callback
-  is assumed to obey
-  ``jax.vmap(callback)(xs) == callback(xs) == jnp.stack([callback(x) for x in xs])``.
-  Therefore, the callback will be called directly on batched inputs (where the
-  batch axes are the leading dimensions). Additionally, the callbacks should
-  return outputs that have corresponding leading batch axes. If not vectorized
-  ``callback`` will be mapped sequentially across the batched axis.
-  For example, if ``callback = lambda x, y: np.matmul(x, y)``, then we are free
-  to set ``vectorized=True`` because the ``np.matmul`` function handles
-  arbitrary leading batch dimensions.
-
-  Args:
-    callback: A Python callable. The callable will be passed PyTrees of NumPy
-      arrays as arguments, and should return a PyTree of NumPy arrays that
-      matches ``result_shape_dtypes``.
-    result_shape_dtypes: A PyTree with leaves that are objects with ``shape``
-      and ``dtype`` attributes which represent to the shapes and dtypes of the
-      value of ``callback`` applied to ``args`` and ``kwargs``.
-    *args: The positional arguments to the callback. Must be PyTrees of JAX
-      types.
-    sharding: optional sharding that specifies the device from which the
-      callback should be invoked.
-    vectorized: A boolean that indicates whether or not ``callback`` is
-      vectorized, meaning it can handle arrays with additional leading
-      dimensions. If ``vectorized`` is `True`, when the callback is mapped
-      via `jax.vmap`, it will be called directly on inputs with leading batch
-      dimensions instead of executing ``callback`` on each mapped input
-      individually. The callback should also return outputs batched across the
-      leading axis. By default, ``vectorized`` is ``False``.
-    **kwargs: The keyword arguments to the callback. Must be PyTrees of JAX
-      types.
-
-  Returns:
-    The value of ``callback(*args, **kwargs)``.
-  """
-  return pure_callback(
-      callback,
-      result_shape_dtypes,
-      *args,
-      sharding=sharding,
-      vectorized=vectorized,
-      **kwargs,
-  )
-
-
 # IO Callback
 
 io_callback_p = core.Primitive("io_callback")
@@ -398,11 +357,14 @@ def io_callback_impl(
     ordered: bool,
 ):
   del result_avals, sharding, ordered
-  try:
-    return callback(*args)
-  except BaseException:
-    logger.exception("jax.io_callback failed")
-    raise
+  cpu_device, *_ = jax.local_devices(backend="cpu")
+  args = jax.device_put(args, cpu_device)
+  with jax.default_device(cpu_device):
+    try:
+      return tree_util.tree_map(np.asarray, callback(*args))
+    except BaseException:
+      logger.exception("jax.io_callback failed")
+      raise
 
 
 io_callback_p.def_impl(functools.partial(dispatch.apply_primitive,
@@ -439,16 +401,16 @@ def io_callback_batching_rule(
 ):
   if ordered:
     raise ValueError("Cannot `vmap` ordered IO callback.")
-  return pure_callback_batching_rule(
-      args,
-      dims,
-      callback=callback,
-      sharding=sharding,
-      vectorized=False,
-      result_avals=result_avals,
-  )
-
-
+  is_batched = [d is not batching.not_mapped for d in dims]
+  new_args = [arg if dim is batching.not_mapped else
+              batching.moveaxis(arg, dim, 0) for arg, dim in zip(args, dims)]
+  unbatched_args, batched_args = util.partition_list(is_batched, new_args)
+  def _batch_fun(batched_args):
+    merged = util.merge_lists(is_batched, unbatched_args, batched_args)
+    return io_callback_p.bind(*merged, callback=callback, sharding=sharding,
+                              result_avals=result_avals, ordered=False)
+  out_vals = lax_map(_batch_fun, batched_args)
+  return out_vals, (0,) * len(out_vals)
 batching.primitive_batchers[io_callback_p] = io_callback_batching_rule
 
 
@@ -474,7 +436,7 @@ def io_callback_lowering(ctx, *args, callback, sharding, ordered, **params):
         list(args),
         ctx.avals_in,
         ctx.avals_out,
-        True,
+        has_side_effect=True,
         sharding=op_sharding,
     )
     ctx.set_tokens_out(mlir.TokenSet({_OrderedIOEffect: (token,)}))
@@ -486,7 +448,7 @@ def io_callback_lowering(ctx, *args, callback, sharding, ordered, **params):
         list(args),
         ctx.avals_in,
         ctx.avals_out,
-        True,
+        has_side_effect=True,
         sharding=op_sharding,
     )
   return result
@@ -508,7 +470,7 @@ def io_callback(
   For more explanation, see `External Callbacks`_.
 
   Args:
-    callback: function to execute on the host. It is assumet to be an impure function.
+    callback: function to execute on the host. It is assumed to be an impure function.
       If ``callback`` is pure, using :func:`jax.pure_callback` instead may lead to
       more efficient execution.
     result_shape_dtypes: pytree whose leaves have ``shape`` and ``dtype`` attributes,
