@@ -33,7 +33,6 @@ from jax._src import profiler
 from jax._src import traceback_util
 from jax._src.interpreters import mlir
 from jax._src.lib import xla_client as xc
-from jax._src.lib import xla_extension_version
 from jax._src.lib.mlir import ir
 from jax._src.xla_bridge import process_count
 import numpy as np
@@ -253,9 +252,7 @@ def compile_or_get_cached(
   # Persistent compilation cache only implemented on TPU and GPU and the backend
   # that supports serialization of executables.
   # TODO(skye): add warning when initializing cache on unsupported default platform
-  supported_platforms = ["tpu", "gpu"]
-  if xla_extension_version >= 253:
-    supported_platforms.append("cpu")
+  supported_platforms = ["tpu", "gpu", "cpu"]
   use_compilation_cache = (
       config.enable_compilation_cache.value
       and getattr(backend, "supports_executable_serialization", True)
@@ -353,6 +350,10 @@ def _compile_and_write_autotune_config(
 ) -> xc.LoadedExecutable:
   share_timeout = config.share_binary_between_hosts_timeout_ms.value
   debug_options = compile_options.executable_build_options.debug_options
+
+  if _compile_and_write_autotune_config.autotune_configs_dir is None:
+    _compile_and_write_autotune_config.autotune_configs_dir = tempfile.mkdtemp()
+
   autotune_tmp_file = os.path.join(
       _compile_and_write_autotune_config.autotune_configs_dir, cache_key
   )
@@ -436,7 +437,7 @@ def _compile_and_write_autotune_config(
     )
   return executable
 
-_compile_and_write_autotune_config.autotune_configs_dir = tempfile.mkdtemp()
+_compile_and_write_autotune_config.autotune_configs_dir = None
 
 # The process with id 0 should compile the module and write it to the K-V
 # storage.
