@@ -772,7 +772,8 @@ def _flatten_bwd(in_tree, in_avals, out_trees, *args):
       results.append(Zero(ct.aval))
     else:
       if (not core.typecompat(a.at_least_vspace(), a_ := core.get_aval(ct))
-          and not _temporary_dtype_exception(a, a_)):
+          and not (_temporary_dtype_exception(a, a_) or
+                   _temporary_shape_exception(a, a_))):
         msg = ("Custom VJP bwd rule must produce an output with the same "
                "shape/dtypes as the args tuple of the primal function, but at "
                f"output{keystr(kp)} the bwd rule produced an output of "
@@ -790,6 +791,9 @@ def _temporary_dtype_exception(a, a_) -> bool:
              dtypes.issubdtype(a.dtype, dtypes.np.inexact)))
   return False
 
+# TODO(mattjj): remove both these exceptions to cotangent compatibility check
+def _temporary_shape_exception(a, a_) -> bool:
+  return config.custom_vjp_disable_shape_check.value
 
 class CustomVJPCallPrimitive(core.CallPrimitive):
   initial_style: core.Primitive
