@@ -35,7 +35,6 @@ from jax._src import util
 from jax._src import xla_bridge
 from jax._src.util import safe_map, safe_zip, use_cpp_class, use_cpp_method
 from jax._src.lib import xla_client as xc
-from jax._src.lib import xla_extension_version
 from jax._src.partition_spec import PartitionSpec
 
 import numpy as np
@@ -136,10 +135,16 @@ class XLACompatibleSharding(sharding.Sharding):
 
 
 @functools.lru_cache
-def _check_mesh_resource_axis(mesh, parsed_pspec):
+def _check_mesh_resource_axis(mesh, parsed_pspec, _manual_axes):
   try:
-    [mesh.shape[r] for p in parsed_pspec if p is not None
-     for r in p]
+    for p in parsed_pspec:
+      if p is not None:
+        for r in p:
+          mesh.shape[r]
+          if r in _manual_axes:
+            raise ValueError(
+                f"Axis: {r} of {parsed_pspec.get_partition_spec()} "
+                f"is also found in manual_axes: {_manual_axes}.") from None
   except KeyError as e:
     raise ValueError(f"Resource axis: {e.args[0]} of {parsed_pspec.user_spec} is "
                      "undefined.") from None
@@ -287,13 +292,6 @@ class NamedSharding(XLACompatibleSharding):
     self._manual_axes = _manual_axes
     self._parsed_pspec = preprocess(self.mesh, self.spec, _parsed_pspec)
 
-  # TODO(phawkins): remove this method when jaxlib 0.4.26 or newer is the
-  # minimum. This method is called by the C++ sharding implementation in earlier
-  # versions.
-  if xla_extension_version < 243:
-    def _preprocess(self):
-      self._parsed_pspec = preprocess(self.mesh, self.spec, self._parsed_pspec)
-
   def __repr__(self):
     mesh_repr = ", ".join(f"'{k}': {v}" for k, v in self.mesh.shape.items())
     mem = '' if self.memory_kind is None else f', memory_kind={self.memory_kind}'
@@ -317,13 +315,13 @@ class NamedSharding(XLACompatibleSharding):
   def __eq__(self, other):
     if not isinstance(other, NamedSharding):
       return False
-    if id(self) == id(other):
+    if self is other:
       return True
     if (self._parsed_pspec != other._parsed_pspec
         or self.memory_kind != other.memory_kind
         or self._manual_axes != other._manual_axes):
       return False
-    return id(self.mesh) == id(other.mesh) or self.mesh == other.mesh
+    return self.mesh is other.mesh or self.mesh == other.mesh
 
   def is_compatible_aval(self, aval_shape: Shape):
     assert self._parsed_pspec is not None
@@ -422,7 +420,7 @@ class SingleDeviceSharding(XLACompatibleSharding):
   def __eq__(self, other):
     if not isinstance(other, SingleDeviceSharding):
       return False
-    if id(self) == id(other):
+    if self is other:
       return True
     return (self._device == other._device and
             self.memory_kind == other.memory_kind)
@@ -485,7 +483,7 @@ class PmapSharding(XLACompatibleSharding):
   def __eq__(self, other):
     if not isinstance(other, PmapSharding):
       return False
-    if id(self) == id(other):
+    if self is other:
       return True
     return (self.sharding_spec == other.sharding_spec and
             self.devices.shape == other.devices.shape and
@@ -673,8 +671,7 @@ class PositionalSharding(XLACompatibleSharding):
 
   def __init__(self, devices: Sequence[xc.Device] | np.ndarray,
                *, memory_kind: str | None = None):
-    if xla_extension_version >= 235:
-      super().__init__()
+    super().__init__()
     if not isinstance(devices, np.ndarray):
       devices = np.array(devices, dtype='object')
     if not devices.size:
@@ -741,12 +738,11 @@ class PositionalSharding(XLACompatibleSharding):
   def __eq__(self, other) -> bool:
     if not isinstance(other, PositionalSharding):
       return False
-    if id(self) == id(other):
+    if self is other:
       return True
     all_ids_equal = np.array_equal(self._ids,other._ids)
     mem_kind_equal = self.memory_kind == other.memory_kind
-    if (id(self._devices) == id(other._devices) and mem_kind_equal and
-        all_ids_equal):
+    if self._devices is other._devices and mem_kind_equal and all_ids_equal:
       return True
     return (mem_kind_equal and all_ids_equal and
             self._internal_device_list == other._internal_device_list)
@@ -852,7 +848,7 @@ class GSPMDSharding(XLACompatibleSharding):
   def __eq__(self, other):
     if not isinstance(other, GSPMDSharding):
       return False
-    if id(self) == id(other):
+    if self is other:
       return True
     return (are_op_shardings_equal(self._hlo_sharding, other._hlo_sharding)
             and self.memory_kind == other.memory_kind
@@ -1106,7 +1102,7 @@ class CanonicalizedParsedPartitionSpec(ParsedPartitionSpec):
             f"sync={self.sync})")
 
 
-def preprocess(mesh, spec, parsed_pspec):
+def preprocess(mesh, spec, parsed_pspec, _manual_axes=frozenset()):
   # This split exists because you can pass `_parsed_pspec` that has been
   # modified from the original. For example: Adding extra dimension to
   # axis_resources for vmap handlers. In such cases you need to preserve the
@@ -1115,13 +1111,15 @@ def preprocess(mesh, spec, parsed_pspec):
   # TODO(yaskatariya): Remove this and replace this with a normalized
   # representation of Parsed Pspec
   if parsed_pspec is None:
-    parsed_pspec, _, _ = prepare_axis_resources(
+    parsed_pspec = prepare_axis_resources(
         PartitionSpec() if spec is None else spec,
         "NamedSharding spec", allow_unconstrained_dims=True)
 
-  _check_mesh_resource_axis(mesh, parsed_pspec)
+  _check_mesh_resource_axis(mesh, parsed_pspec, _manual_axes)
   return parsed_pspec
 
+# fallback for c++ .
+preprocess_with_manual = preprocess
 
 def prepare_axis_resources(axis_resources,
                            arg_name,
@@ -1148,7 +1146,7 @@ def prepare_axis_resources(axis_resources,
           entry, what, allow_unconstrained_dims=allow_unconstrained_dims))
 
   _check_unique_resources(new_entries, arg_name)
-  return tree_util.tree_unflatten(treedef, new_entries), new_entries, treedef
+  return tree_util.tree_unflatten(treedef, new_entries)
 
 
 def _check_unique_resources(axis_resources, arg_name):
