@@ -44,11 +44,10 @@ from jax._src import traceback_util
 from jax._src import tree_util
 from jax._src.tree_util import tree_unflatten, keystr
 from jax._src import util
-from jax._src.layout import DeviceLocalLayout
+from jax._src.layout import Layout
 from jax._src.interpreters import mlir
 from jax._src.lib.mlir import ir
 from jax._src.lib import xla_client as xc
-from jax._src.lib import xla_extension_version
 
 
 source_info_util.register_exclusion(__file__)
@@ -88,12 +87,10 @@ class Executable(Protocol):
     """
     raise NotImplementedError
 
-  # Layouts are exposed via jax.experimental.layouts
-  # TODO(frostig,yashkatariya): expose here when no longer experimental.
-  def _input_layouts(self):
+  def input_layouts(self):
     raise NotImplementedError
 
-  def _output_layouts(self):
+  def output_layouts(self):
     raise NotImplementedError
 
   def as_text(self) -> str:
@@ -228,11 +225,11 @@ class XlaExecutable(Executable):
     raise NotImplementedError(
         "compiled executable carries no output sharding information")
 
-  def _input_layouts(self):
+  def input_layouts(self):
     raise NotImplementedError(
         "compiled executable carries no input layout information")
 
-  def _output_layouts(self):
+  def output_layouts(self):
     raise NotImplementedError(
         "compiled executable carries no input layout information")
 
@@ -317,10 +314,7 @@ class XlaLowering(Lowering):
     """Return an HLO representation of this computation."""
     hlo = self.stablehlo()
     m: Union[str, bytes]
-    if xla_extension_version >= 244:
-      m = mlir.module_to_bytecode(hlo)
-    else:
-      m = mlir.module_to_string(hlo)
+    m = mlir.module_to_bytecode(hlo)
     return xla_extension.mlir.mlir_module_to_xla_computation(
         m, use_tuple_args=self.compile_args["tuple_args"])
 
@@ -511,19 +505,19 @@ class Compiled(Stage):
     shardings_flat = self._executable.output_shardings()
     return tree_util.tree_unflatten(self.out_tree, shardings_flat)  # pytype: disable=attribute-error
 
-  def _input_layouts(self):
+  def input_layouts(self):
     layouts_flat = self._executable.input_layouts()
-    assert all(isinstance(l, DeviceLocalLayout) for l in layouts_flat)
+    assert all(isinstance(l, Layout) for l in layouts_flat)
     # Some input layouts got DCE'd
     if self.in_tree.num_leaves > len(layouts_flat):
       iter_layouts_flat = iter(layouts_flat)
       layouts_flat = [next(iter_layouts_flat) if i in self._executable._kept_var_idx
-                      else None for i in range(self.in_tree.num_leaves)]
+                      else Layout() for i in range(self.in_tree.num_leaves)]
     return tree_util.tree_unflatten(self.in_tree, layouts_flat)  # pytype: disable=attribute-error
 
-  def _output_layouts(self):
+  def output_layouts(self):
     layouts_flat = self._executable.output_layouts()
-    assert all(isinstance(l, DeviceLocalLayout) for l in layouts_flat)
+    assert all(isinstance(l, Layout) for l in layouts_flat)
     return tree_util.tree_unflatten(self.out_tree, layouts_flat)  # pytype: disable=attribute-error
 
   @staticmethod
