@@ -57,7 +57,7 @@ from jax._src.numpy.util import promote_dtypes, promote_dtypes_inexact
 from jax._src.util import unzip2
 from jax._src.public_test_util import (  # noqa: F401
     _assert_numpy_allclose, _check_dtypes_match, _default_tolerance, _dtype, check_close, check_grads,
-    check_jvp, check_vjp, default_gradient_tolerance, default_tolerance, tolerance)
+    check_jvp, check_vjp, default_gradient_tolerance, default_tolerance, tolerance, rand_like)
 from jax._src import xla_bridge
 
 
@@ -419,6 +419,12 @@ def is_device_tpu(version: int | None = None, variant: str = "") -> bool:
     return "v5 lite" in device_kind
   return expected_version in device_kind
 
+def is_cuda_compute_capability_at_least(capability: str) -> bool:
+  if not is_device_cuda():
+    return False
+  d, *_ = jax.local_devices(backend="gpu")
+  return d.compute_capability >= capability
+
 def _get_device_tags():
   """returns a set of tags defined for the device under test"""
   if is_device_rocm():
@@ -1289,7 +1295,8 @@ class _LazyDtypes:
 
   @_cached_property
   def all_integer(self):
-    return self.supported([np.int8, np.int16, np.int32, np.int64])
+    return self.supported([
+        _dtypes.int4, np.int8, np.int16, np.int32, np.int64])
 
   @_cached_property
   def unsigned(self):
@@ -1297,7 +1304,8 @@ class _LazyDtypes:
 
   @_cached_property
   def all_unsigned(self):
-    return self.supported([np.uint8, np.uint16, np.uint32, np.uint64])
+    return self.supported([
+        _dtypes.uint4, np.uint8, np.uint16, np.uint32, np.uint64])
 
   @_cached_property
   def complex(self):
@@ -1725,22 +1733,12 @@ class numpy_with_mpmath:
 
   def sqrt(self, x):
     ctx = x.context
-    # workaround mpmath bugs:
     if isinstance(x, ctx.mpc):
-      if ctx.isinf(x.real) and ctx.isinf(x.imag):
-        if x.real > 0: return x
-        ninf = x.real
-        inf = -ninf
-        if x.imag > 0: return ctx.make_mpc((inf._mpf_, inf._mpf_))
-        return ctx.make_mpc((inf._mpf_, inf._mpf_))
-      elif ctx.isfinite(x.real) and ctx.isinf(x.imag):
-        if x.imag > 0:
-          inf = x.imag
-          return ctx.make_mpc((inf._mpf_, inf._mpf_))
-        else:
-          ninf = x.imag
-          inf = -ninf
-          return ctx.make_mpc((inf._mpf_, ninf._mpf_))
+      # Workaround mpmath 1.3 bug in sqrt(+-inf+-infj) evaluation (see mpmath/mpmath#776).
+      # TODO(pearu): remove this function when mpmath 1.4 or newer
+      # will be the required test dependency.
+      if ctx.isinf(x.imag):
+        return ctx.make_mpc((ctx.inf._mpf_, x.imag._mpf_))
     return ctx.sqrt(x)
 
   def expm1(self, x):
@@ -1764,6 +1762,34 @@ class numpy_with_mpmath:
           return ctx.make_mpc(((-x.real)._mpf_, (3 * pi / 4)._mpf_))
     return ctx.log1p(x)
 
+  def tan(self, x):
+    ctx = x.context
+    if isinstance(x, ctx.mpc):
+      # Workaround mpmath 1.3 bug in tan(+-inf+-infj) evaluation (see mpmath/mpmath#781).
+      # TODO(pearu): remove this function when mpmath 1.4 or newer
+      # will be the required test dependency.
+      if ctx.isinf(x.imag) and (ctx.isinf(x.real) or ctx.isfinite(x.real)):
+        if x.imag > 0:
+          return ctx.make_mpc((ctx.zero._mpf_, ctx.one._mpf_))
+        return ctx.make_mpc((ctx.zero._mpf_, (-ctx.one)._mpf_))
+      if ctx.isinf(x.real) and ctx.isfinite(x.imag):
+        return ctx.make_mpc((ctx.nan._mpf_, ctx.nan._mpf_))
+    return ctx.tan(x)
+
+  def tanh(self, x):
+    ctx = x.context
+    if isinstance(x, ctx.mpc):
+      # Workaround mpmath 1.3 bug in tanh(+-inf+-infj) evaluation (see mpmath/mpmath#781).
+      # TODO(pearu): remove this function when mpmath 1.4 or newer
+      # will be the required test dependency.
+      if ctx.isinf(x.imag) and (ctx.isinf(x.real) or ctx.isfinite(x.real)):
+        if x.imag > 0:
+          return ctx.make_mpc((ctx.zero._mpf_, ctx.one._mpf_))
+        return ctx.make_mpc((ctx.zero._mpf_, (-ctx.one)._mpf_))
+      if ctx.isinf(x.real) and ctx.isfinite(x.imag):
+        return ctx.make_mpc((ctx.nan._mpf_, ctx.nan._mpf_))
+    return ctx.tanh(x)
+
   def log2(self, x):
     return x.context.ln(x) / x.context.ln2
 
@@ -1773,6 +1799,60 @@ class numpy_with_mpmath:
   def exp2(self, x):
     return x.context.exp(x * x.context.ln2)
 
+  def arcsin(self, x):
+    ctx = x.context
+    if isinstance(x, ctx.mpc):
+      # Workaround mpmath 1.3 bug in asin(+-inf+-infj) evaluation (see
+      # mpmath/mpmath#793).
+      # TODO(pearu): remove the if-block below when mpmath 1.4 or
+      # newer will be the required test dependency.
+      pi = ctx.pi
+      inf = ctx.inf
+      zero = ctx.zero
+      if ctx.isinf(x.real):
+        sign_real = -1 if x.real < 0 else 1
+        real = sign_real * pi / (4 if ctx.isinf(x.imag) else 2)
+        imag = -inf if x.imag < 0 else inf
+        return ctx.make_mpc((real._mpf_, imag._mpf_))
+      elif ctx.isinf(x.imag):
+        return ctx.make_mpc((zero._mpf_, x.imag._mpf_))
+
+      # On branch cut, mpmath.mp.asin returns different value compared
+      # to mpmath.fp.asin and numpy.arcsin (see
+      # mpmath/mpmath#786). The following if-block ensures
+      # compatibiliy with numpy.arcsin.
+      if x.real > 1 and x.imag == 0:
+        return ctx.asin(x).conjugate()
+
+    return ctx.asin(x)
+
+  def arcsinh(self, x):
+    ctx = x.context
+
+    if isinstance(x, ctx.mpc):
+      # Workaround mpmath 1.3 bug in asinh(+-inf+-infj) evaluation
+      # (see mpmath/mpmath#749).
+      # TODO(pearu): remove the if-block below when mpmath 1.4 or
+      # newer will be the required test dependency.
+      pi = ctx.pi
+      inf = ctx.inf
+      zero = ctx.zero
+      if ctx.isinf(x.imag):
+        sign_imag = -1 if x.imag < 0 else 1
+        real = -inf if x.real < 0 else inf
+        imag = sign_imag * pi / (4 if ctx.isinf(x.real) else 2)
+        return ctx.make_mpc((real._mpf_, imag._mpf_))
+      elif ctx.isinf(x.real):
+        return ctx.make_mpc((x.real._mpf_, zero._mpf_))
+
+      # On branch cut, mpmath.mp.asinh returns different value
+      # compared to mpmath.fp.asinh and numpy.arcsinh (see
+      # mpmath/mpmath#786).  The following if-block ensures
+      # compatibiliy with numpy.arcsinh.
+      if x.real == 0 and x.imag < -1:
+        return (-ctx.asinh(x)).conjugate()
+    return ctx.asinh(x)
+
   def normalize(self, exact, reference, value):
     """Normalize reference and value using precision defined by the
     difference of exact and reference.
