@@ -26,7 +26,6 @@ import os
 import time
 from typing import Any, Callable
 
-from absl import flags
 import jax
 from jax import core
 from jax._src import config
@@ -42,7 +41,11 @@ from jaxlib.mlir.dialects import stablehlo
 from jaxlib.mlir.passmanager import PassManager
 import numpy as np
 
-FLAGS = flags.FLAGS
+try:
+  from absl import flags
+  FLAGS = flags.FLAGS
+except ImportError:
+  FLAGS = {}
 
 _MOSAIC_USE_PYTHON_PIPELINE = config.define_bool_state(
     name="mosaic_use_python_pipeline",
@@ -321,7 +324,12 @@ def _lower_tpu_kernel(
     pipeline.run(module.operation)
     dump_mlir(module, "post-simplify")
 
-    if checks := FLAGS["xla_mosaic_on_device_checks"].value:
+    try:
+      on_device_checks = FLAGS["xla_mosaic_on_device_checks"].value
+    except KeyError:
+      on_device_checks = False
+
+    if checks := on_device_checks:
       checks = set(checks.split(","))
       if checks == {"bounds"}:  # We only support one kind of checks now.
         pipeline = PassManager.parse(
@@ -389,6 +397,7 @@ def as_tpu_kernel(
   has_communication, has_custom_barrier = tpu.private_has_communication(
       module.operation
   )
+  needs_hlo_passes = _MOSAIC_ALLOW_HLO.value
   needs_layout_passes = not device_type
   # We'll mutate the module, so clone it
   with module.context as ctx, module.operation.location as _:
@@ -397,6 +406,7 @@ def as_tpu_kernel(
     )
     if needs_layout_passes and _MOSAIC_USE_PYTHON_PIPELINE.value:
       module = _lower_tpu_kernel(module, hardware_generation)
+      needs_hlo_passes = False
       needs_layout_passes = False
     prev_allow_unregistered_dialects = ctx.allow_unregistered_dialects
     ctx.allow_unregistered_dialects = True
@@ -415,7 +425,7 @@ def as_tpu_kernel(
   return _lowered_as_tpu_kernel(
       asm,
       out_type,
-      needs_hlo_passes=_MOSAIC_ALLOW_HLO.value,
+      needs_hlo_passes=needs_hlo_passes,
       needs_layout_passes=needs_layout_passes,
       device_type=device_type,
       has_communication=has_communication,
