@@ -22,6 +22,7 @@ from __future__ import annotations
 
 from __future__ import annotations
 
+import atexit
 from collections.abc import Mapping
 import dataclasses
 from functools import lru_cache, partial
@@ -46,7 +47,6 @@ from jax._src.cloud_tpu_init import maybe_import_libtpu
 from jax._src.lib import cuda_versions
 from jax._src.lib import xla_client
 from jax._src.lib import xla_extension
-from jax._src.lib import xla_extension_version
 from jax._src.lib import jaxlib
 
 logger = logging.getLogger(__name__)
@@ -103,7 +103,23 @@ _MOCK_NUM_GPUS = config.DEFINE_integer(
 _CPU_ENABLE_GLOO_COLLECTIVES = config.DEFINE_bool(
     name="jax_cpu_enable_gloo_collectives",
     default=False,
-    help="If True, enable cross-process collectives on CPU using Gloo.",
+    help="Deprecated, please use jax_cpu_collectives_implementation instead.",
+)
+
+_CPU_COLLECTIVES_IMPLEMENTATION = config.DEFINE_string(
+    name='jax_cpu_collectives_implementation',
+    default='none',
+    help='Cross-process collective implementation used on CPU. Either "none", '
+         '"gloo" or "mpi"'
+)
+
+# TODO(yueshengys): turn default back to True after resolving memory increase
+# issue.
+_CPU_ENABLE_ASYNC_DISPATCH = config.DEFINE_bool(
+    name="jax_cpu_enable_async_dispatch",
+    default=False,
+    help="Only applies to non-parallel computations. If False, run computations"
+    "inline without async dispatch.",
 )
 
 
@@ -114,10 +130,7 @@ def _at_fork():
     "and JAX is multithreaded, so this will likely lead to a deadlock.",
     RuntimeWarning, stacklevel=2)
 
-# os.register_at_fork only exists on Unix.
-if hasattr(os, "register_at_fork"):
-  os.register_at_fork(before=_at_fork)
-
+_at_fork_handler_installed = False
 
 # Backends
 
@@ -129,10 +142,6 @@ def _get_tpu_library_path() -> str | None:
 
   libtpu_module = maybe_import_libtpu()
   if libtpu_module is not None:
-    if xla_extension_version < 212:
-      # xla_extension_version < 212 uses tpu_tracer which requires calling
-      # configure_library_path.
-      libtpu_module.configure_library_path()
     return libtpu_module.get_library_path()
 
   return None
@@ -226,28 +235,36 @@ def register_backend_factory(name: str, factory: BackendFactory, *,
 
 
 def make_cpu_client() -> xla_client.Client:
-  if xla_extension_version >= 223:
-    collectives: xla_client._xla.CpuCollectives | None = None
-    if _CPU_ENABLE_GLOO_COLLECTIVES.value:
-      collectives = xla_client._xla.make_gloo_tcp_collectives(  # type: ignore
-        distributed_client=distributed.global_state.client,
-      )
-    return xla_client.make_cpu_client(  # type: ignore
+  collectives: xla_client._xla.CpuCollectives | None = None
+
+  collectives_impl = _CPU_COLLECTIVES_IMPLEMENTATION.value
+  if _CPU_ENABLE_GLOO_COLLECTIVES.value:
+      collectives_impl = 'gloo'
+      warnings.warn('Setting `jax_cpu_enable_gloo_collectives` is deprecated. '
+                    'Please use `jax.config.update('
+                    '"jax_cpu_collectives_implementation", "gloo")` instead.',
+                    DeprecationWarning,
+                    )
+  if collectives_impl == 'gloo':
+    collectives = xla_client._xla.make_gloo_tcp_collectives(  # type: ignore
       distributed_client=distributed.global_state.client,
-      node_id=distributed.global_state.process_id,
-      num_nodes=distributed.global_state.num_processes,
-      collectives=collectives,
     )
-  elif xla_extension_version >= 216:
-    # TODO(phawkins): remove type: ignore after updating jaxlib version used for
-    # mypy checks.
-    return xla_client.make_cpu_client(  # type: ignore
-      distributed_client=distributed.global_state.client,
-      node_id=distributed.global_state.process_id,
-      num_nodes=distributed.global_state.num_processes,
-    )
-  else:
-    return xla_client.make_cpu_client()
+  elif collectives_impl == 'mpi':
+    collectives = xla_client._xla.make_mpi_collectives()  # type: ignore
+    collectives.Init()  # type: ignore
+    atexit.register(collectives.Finalize)  # type: ignore
+  elif collectives_impl != 'none':
+    collectives_impls = ['none', 'gloo', 'mpi']
+    raise RuntimeError(f"Unknown collectives implementation "
+                       f"{collectives_impl}. Available implementations are "
+                       f"{collectives_impls}.")
+  return xla_client.make_cpu_client(  # type: ignore
+    asynchronous=_CPU_ENABLE_ASYNC_DISPATCH.value,
+    distributed_client=distributed.global_state.client,
+    node_id=distributed.global_state.process_id,
+    num_nodes=distributed.global_state.num_processes,
+    collectives=collectives,
+  )
 
 
 register_backend_factory(
@@ -677,13 +694,10 @@ def register_plugin(
     c_api = xla_client.load_pjrt_plugin_dynamically(plugin_name, library_path)  # type: ignore
     xla_client.profiler.register_plugin_profiler(c_api)
   else:
-    if xla_extension_version >= 236:
-      assert c_api is not None
-      xla_client.load_pjrt_plugin_with_c_api(plugin_name, c_api)
-  if xla_extension_version >= 239:
-    make_topology = partial(xla_client.make_c_api_device_topology, c_api)
-  else:
-    make_topology = None
+    assert c_api is not None
+    xla_client.load_pjrt_plugin_with_c_api(plugin_name, c_api)
+
+  make_topology = partial(xla_client.make_c_api_device_topology, c_api)
   experimental = plugin_name not in _nonexperimental_plugins
   register_backend_factory(plugin_name, factory, priority=priority,
                            fail_quietly=False, experimental=experimental,
@@ -824,12 +838,19 @@ def backends() -> dict[str, xla_client.Client]:
   global _backends
   global _backend_errors
   global _default_backend
+  global _at_fork_handler_installed
 
   _discover_and_register_pjrt_plugins()
 
   with _backend_lock:
     if _backends:
       return _backends
+
+    # os.register_at_fork only exists on Unix.
+    if not _at_fork_handler_installed and hasattr(os, "register_at_fork"):
+      os.register_at_fork(before=_at_fork)
+      _at_fork_handler_installed = True
+
     if jax_platforms := config.jax_platforms.value:
       platforms = []
       # Allow platform aliases in the list of platforms.
@@ -895,8 +916,17 @@ def _suggest_missing_backends():
       any(os.path.exists(d) for d in nvidia_gpu_devices)):
     if hasattr(xla_extension, "GpuAllocatorConfig") and "cuda" in _backend_errors:
       err = _backend_errors["cuda"]
-      logger.warning(f"CUDA backend failed to initialize: {err} (Set "
-                     "TF_CPP_MIN_LOG_LEVEL=0 and rerun for more info.)")
+      warning_msg = f"CUDA backend failed to initialize: {err}."
+      if "no supported devices found for platform CUDA." in err:
+        warning_msg += (
+          "This may be due to JAX pre-allocating too much device "
+          "memory, leaving too little for CUDA library initialization. See "
+          "https://jax.readthedocs.io/en/latest/gpu_memory_allocation.html "
+          "for more details and potential workarounds."
+        )
+      warning_msg += "(Set TF_CPP_MIN_LOG_LEVEL=0 and rerun for more info.)"
+
+      logger.warning(warning_msg)
     else:
       logger.warning("An NVIDIA GPU may be present on this machine, but a "
                      "CUDA-enabled jaxlib is not installed. Falling back to "
@@ -1065,6 +1095,18 @@ def backend_pjrt_c_api_version(platform=None) -> tuple[int, int] | None:
   return None
 
 
+def backend_xla_version(platform=None) -> int | None:
+  """Returns the XLA version of the backend.
+
+  Returns None if the backend does not use PJRT C API or does not have
+  xla_version in the plugin attributes. This methon can be used to skip features
+  that are not available before certain xla_version if the backend is a
+  plugin and uses xla_version.
+  """
+  backend = get_backend(platform)
+  return getattr(backend, "xla_version", None)
+
+
 @lru_cache
 def local_devices(process_index: int | None = None,
                   backend: str | xla_client.Client | None = None,
