@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from jax.experimental.array_api._linear_algebra_functions import (
+from jax.numpy.linalg import (
   cholesky as cholesky,
   cross as cross,
   det as det,
@@ -23,17 +23,22 @@ from jax.experimental.array_api._linear_algebra_functions import (
   matmul as matmul,
   matrix_norm as matrix_norm,
   matrix_power as matrix_power,
-  matrix_rank as matrix_rank,
   matrix_transpose as matrix_transpose,
   outer as outer,
-  pinv as pinv,
   qr as qr,
   slogdet as slogdet,
   solve as solve,
   svd as svd,
   svdvals as svdvals,
   tensordot as tensordot,
-  trace as trace,
   vecdot as vecdot,
   vector_norm as vector_norm,
 )
+
+# TODO(micky774): Add trace to jax.numpy.linalg
+from jax.numpy import trace as trace
+
+from jax.experimental.array_api._linear_algebra_functions import (
+  matrix_rank as matrix_rank,
+  pinv as pinv,
+)
